Attribute VB_Name = "FedExAPI"
Dim fdxInitialized As Boolean
Dim fdxApiEnabled As Boolean

Dim fdxDir As String, fdxIni As String

Dim fdxApiUsername As String
Dim fdxApiPassword As String
Dim fdxApiKey As String

Dim fdxRate As FedExTransaction

Dim fdxStr(10) As String, fdxInt(10) As Integer, fdxDbl(10) As Double, fdxBool(10) As Boolean

Type FedExService
    Code As String
    Type As String
    Description As String
    LTL As Boolean
End Type

Type FedExAddress
    AccountNumber As String
    CompanyName As String
    ContactName As String
    AddressLine1 As String
    AddressLine2 As String
    AddressLine3 As String
    City As String
    State As String
    PostalCode As String
    CountryCode As String
    Phone As String
    EmailAddress As String
    Residential As Boolean
End Type

Type FedExTransaction
    Request As String
    Response As String
End Type

Public Enum FedExRateType
    Base
    Total
End Enum

Type FedExRateResult
    Status As String
    ErrorCode As Integer
    ErrorMessage As String
    Success As Boolean
    Base As Double
    Total As Double
    Fuel As Double
    LiftGate As Double
    InsideDelivery As Double
    SurchargeTotal As Double
End Type

Function Initialize()
    Initialize = fdxInitialized
    If fdxInitialized Then
        Exit Function
    End If
    
    fdxDir = CS32.ShipDir & "Logic\FedEx"
    
    If Dir(fdxDir, vbDirectory) = "" Then
        MkDir fdxDir
    End If
    fdxDir = fdxDir & "\"
        
    If Dir(fdxDir & "XML", vbDirectory) = "" Then
        MkDir fdxDir & "XML"
    End If
    
    fdxTmpDir = RunDir & "FedEx"
    If Dir(fdxTmpDir, vbDirectory) = "" Then
        MkDir fdxTmpDir
    End If
    fdxTmpDir = fdxTmpDir & "\"
    
    If Dir(fdxTmpDir & "Temp", vbDirectory) = "" Then
        MkDir fdxTmpDir & "Temp"
    End If
    
    fdxIni = fdxDir & "FedEx.ini"
    
    fdxStr(0) = Left(Upper(IniRead(fdxIni, "API", "Enabled", "")), 1)
    If KiEmpty(fdxStr(0)) Then
        fdxStr(0) = "N"
        IniWrite fdxIni, "API", "Enabled", fdxStr(0)
    End If
    fdxApiEnabled = (fdxStr(0) = "Y")
    
    If fdxApiEnabled Then
        fdxApiKey = IniRead(fdxIni, "API", "Key", "")
        fdxApiPassword = IniRead(fdxIni, "API", "Password", "")
        
        If KiEmpty(fdxApiPassword) Or KiEmpty(fdxApiKey) Then
            MsgBox "FedEx Web Services settings missing. FWS will be disabled.", vbInformation
            fdxApiEnabled = False
        End If
    End If
    
    fdxInitialized = True
    Initialize = fdxInitialized
End Function

Private Function CreateRateRequest_Freight(ByRef FdxSvc As FedExService) As String
    Dim sRequestXml As String
    Dim iCsAcct As Integer
    iCsAcct = CInt(CS32.GetVar("xACCOUNT"))
    
    sRequestXml = "<soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" " & _
        "xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" " & _
        "xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">" & vbLf & _
        vbTab & "<soap:Body>" & vbLf
        
    sRequestXml = sRequestXml & _
        vbTab & vbTab & "<RateRequest xmlns=""http://fedex.com/ws/rate/v22"">" & vbLf & _
        vbTab & vbTab & vbTab & "<WebAuthenticationDetail>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<UserCredential>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Key", IniRead(fdxIni, "Account" & iCsAcct, "APIKey", IniRead(fdxIni, "API", "Key", ""))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Password", IniRead(fdxIni, "Account" & iCsAcct, "APIPassword", IniRead(fdxIni, "API", "Password", ""))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</UserCredential>" & vbLf & _
        vbTab & vbTab & vbTab & "</WebAuthenticationDetail>" & vbLf & _
        vbTab & vbTab & vbTab & "<ClientDetail>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccountNumber", IniRead(fdxIni, "Account" & iCsAcct, "AccountNumber", CS32.GetVar("xACCTNUMFDX"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("MeterNumber", IniRead(fdxIni, "Account" & iCsAcct, "MeterNumber", CS32.GetVar("xFDXPWS_NO"))) & vbLf & _
        vbTab & vbTab & vbTab & "</ClientDetail>" & vbLf & _
        vbTab & vbTab & vbTab & "<TransactionDetail>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CustomerTransactionId", "Rate-" & Trim(CS32.GetVar("xPKGID"))) & vbLf & _
        vbTab & vbTab & vbTab & "</TransactionDetail>" & vbLf

    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & "<Version>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ServiceId", "crs") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Major", "22") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Intermediate", "0") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Minor", "0") & vbLf & _
        vbTab & vbTab & vbTab & "</Version>" & vbLf

    fdxInt(1) = CInt(CS32.GetVar("xACCOUNT"))
    
    'Lookup Shipper info.
    Dim shipper As FedExAddress
    shipper.CompanyName = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperCompanyName", "")
    shipper.AddressLine1 = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperAddress1", "")
    shipper.City = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperCity", "")
    shipper.State = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperState", "")
    shipper.PostalCode = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperZip", "")
    shipper.CountryCode = IniRead(fdxIni, "Account" & fdxInt(1), "ShipperCountry", "US")

    fdxBool(1) = KiEmpty(shipper.AccountNumber)
    fdxBool(2) = KiEmpty(shipper.CompanyName & shipper.ContactName & shipper.AddressLine1 & shipper.City & shipper.State & shipper.PostalCode)
    If fdxBool(1) Or fdxBool(2) Then
        Dim kdvAcctInfo As KiDataView
        Set kdvAcctInfo = DbfSelect(CS32.ShipDir & "CS_ACCT.FIL", "RECNO()=" & fdxInt(1))
        If Not kdvAcctInfo.EOF Then
            If fdxBool(1) Then
                shipper.AccountNumber = Trim(kdvAcctInfo("ACCTNUMFDX"))
            End If
            If fdxBool(2) Then
                shipper.CompanyName = Trim(kdvAcctInfo("FROM1"))
                shipper.AddressLine1 = Trim(kdvAcctInfo("FROM2"))
                shipper.City = Trim(kdvAcctInfo("FROMCITY"))
                shipper.State = Trim(kdvAcctInfo("FROMSTATE"))
                shipper.PostalCode = Trim(kdvAcctInfo("FROMZIP"))
            End If
        End If
        Set kdvAcctInfo = Nothing
    End If
    
    'Build Shipper Xml.
    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & "<RequestedShipment>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ServiceType", FdxSvc.Type) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PackagingType", "YOUR_PACKAGING") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Shipper>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StreetLines", shipper.AddressLine1) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", shipper.City) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateOrProvinceCode", shipper.State) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", shipper.PostalCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", shipper.CountryCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Residential", CInt(shipper.Residential)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Shipper>" & vbLf
        
    'Build Recipient Xml.
    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & vbTab & "<Recipient>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StreetLines", Trim(CS32.GetVar("xSHIPTO4"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StreetLines", Trim(CS32.GetVar("xSHIPTO5"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StreetLines", Trim(CS32.GetVar("xSHIPTO6"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", GetCity(Trim(CS32.GetVar("xSHIPTO7")))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateOrProvinceCode", GetState(Trim(CS32.GetVar("xSHIPTO7")))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Trim(CS32.GetVar("xZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Residential", "0") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Recipient>" & vbLf
        
    'Look up billto.
    Dim billto As FedExAddress
    billto.AccountNumber = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingAccountNumber", "")
    billto.CompanyName = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingCompany", "")
    billto.ContactName = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingContact", "")
    billto.AddressLine1 = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingAddress1", "")
    billto.AddressLine2 = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingAddress2", "")
    billto.AddressLine3 = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingAddress3", "")
    billto.City = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingCity", "")
    billto.State = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingState", "")
    billto.PostalCode = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingZip", "")
    billto.CountryCode = IniRead(fdxIni, "Account" & fdxInt(1), "FreightBillingCountry", "US")
    
    'Build Payment Xml.
    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & vbTab & "<ShippingChargesPayment>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PaymentType", "SENDER") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Payor>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<ResponsibleParty>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccountNumber", billto.AccountNumber) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</ResponsibleParty>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Payor>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</ShippingChargesPayment>" & vbLf
        
    Dim kdvBlData As KiDataView
    Set kdvBlData = DbfSelect(ProjectDir & "BL_DATA.DBF", , , 0)
    
    If Not kdvBlData.EOF Then
        'Build special services.
        fdxStr(3) = Trim(kdvBlData("SS_FLAGS"))
        If (At("Y", fdxStr(3)) > 0) Then
            sRequestXml = sRequestXml & _
                vbTab & vbTab & vbTab & vbTab & vbTab & "<SpecialServicesRequested>" & vbLf
            
            If SubStr(fdxStr(3), 1, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "CALL_BEFORE_DELIVERY") & vbLf
            End If
            If SubStr(fdxStr(3), 4, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "LIMITED_ACCESS_DELIVERY") & vbLf
            End If
            If SubStr(fdxStr(3), 10, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "DO_NOT_STACK_PALLETS") & vbLf
            End If
            If SubStr(fdxStr(3), 16, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "LIFTGATE_DELIVERY") & vbLf
            End If
            'If SubStr(fdxStr(3), 12, 1) = "Y" Then
            '    sRequestXml = sRequestXml & _
            '        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "RESIDENTIAL_DELIVERY") & vbLf
            'End If
            If SubStr(fdxStr(3), 13, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "DO_NOT_BREAK_DOWN_PALLETS") & vbLf
            End If
            If SubStr(fdxStr(3), 18, 1) = "Y" Then
                sRequestXml = sRequestXml & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SpecialServiceTypes", "INSIDE_DELIVERY") & vbLf
            End If
            sRequestXml = sRequestXml & _
                vbTab & vbTab & vbTab & vbTab & vbTab & "</SpecialServicesRequested>" & vbLf
        End If
    End If
    
    'Build Freight details Xml.
    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & vbTab & "<FreightShipmentDetail>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("FedExFreightAccountNumber", billto.AccountNumber) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<FedExFreightBillingContactAndAddress>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Contact>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PersonName", billto.ContactName) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CompanyName", billto.CompanyName) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PhoneNumber", billto.Phone) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Contact>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StreetLines", billto.AddressLine1) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", billto.City) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateOrProvinceCode", billto.State) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", billto.PostalCode) & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", billto.CountryCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</FedExFreightBillingContactAndAddress>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Role", "SHIPPER") & vbLf

    While Not kdvBlData.EOF
    'Add line items
        sRequestXml = sRequestXml & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<LineItems>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("FreightClass", FedExClass(kdvBlData("CLASS"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", Trim(kdvBlData("ITEMDESC"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Weight>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Units", "LB") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Value", Trim(kdvBlData("WEIGHT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Weight>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</LineItems>" & vbLf
        kdvBlData.MoveNext
    Wend
    
    Set kdvBlData = Nothing
    
    sRequestXml = sRequestXml & _
        vbTab & vbTab & vbTab & vbTab & "</FreightShipmentDetail>" & vbLf & _
        vbTab & vbTab & vbTab & "</RequestedShipment>" & vbLf & _
        vbTab & vbTab & "</RateRequest>" & vbLf & _
        vbTab & "</soap:Body>" & vbLf & _
        "</soap:Envelope>" & vbLf
        
    CreateRateRequest_Freight = sRequestXml
End Function

Function FedExClass(ByVal class As String) As String
    'Valid FedEx classes are; CLASS_070, CLASS_077_5, CLASS_150, etc.
    FedExClass = Str(Val(class), 5, 1)
    FedExClass = StrTran(FedExClass, ".0", "")
    FedExClass = StrTran(FedExClass, ".", "_")
    FedExClass = StrTran(FedExClass, " ", "0")
    FedExClass = "CLASS_" & FedExClass
End Function

Function Check(ByVal sCsIntcode As String) As Boolean
    Check = False
    
    If KiEmpty(FedExAPI.GetService("US", sCsIntcode, "RATE").Type) Then
        Exit Function
    End If
    
    Check = True
End Function

Function GetRate(Optional ByRef ReturnRate As FedExRateType = FedExRateType.Total, Optional ByRef ShowStatus As Boolean = True) As FedExRateResult
    fdxInitialized = Initialize()
    If Not fdxInitialized Then
        Exit Function
    End If
    
    GetRate.Success = False
    
    Dim FdxSvc As FedExService
    FdxSvc = FedExAPI.GetService("US", CS32.Intcode, "RATE")
    If KiEmpty(FdxSvc.Code) Then
        Exit Function
    End If
    
    If ShowStatus Then PopStatus "FedEx Web Services", "Getting rates for FedEx Freight, please wait..."
    
    fdxStr(0) = ""
    If FdxSvc.LTL Then
       fdxStr(0) = CreateRateRequest_Freight(FdxSvc)
    Else
       'fdxStr(0) = CreateRateRequest
    End If
    
    If fdxRate.Request <> fdxStr(0) Then
        fdxRate.Response = ""
    End If
        
    fdxStr(1) = Xml.ReadXPathFromText(fdxRate.Response, "/RateReply/Notifications/Code/text()")
    If (fdxStr(1) <> "0") Then
        fdxRate.Request = fdxStr(0)
        fdxRate.Response = SendRequest(fdxRate.Request, "Rate")
    End If
    KillStatus

    GetRate.Status = Xml.ReadXPathFromText(fdxRate.Response, "/RateReply/Notifications/Severity/text()")
    If (GetRate.Status = "SUCCESS") Then
        fdxStr(2) = "/RateReply/RateReplyDetails/RatedShipmentDetails/ShipmentRateDetail/"
    
        GetRate.Success = True
        GetRate.Base = Val(Trim(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "TotalNetFreight/Amount/text()")))
        GetRate.Total = Val(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "TotalNetFedExCharge/Amount/text()"))
        GetRate.SurchargeTotal = Val(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "TotalSurcharges/Amount/text()"))
        GetRate.Fuel = Val(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "Surcharges[SurchargeType='FUEL']/Amount/Amount/text()"))
        GetRate.LiftGate = Val(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "Surcharges[SurchargeType='LIFTGATE_DELIVERY']/Amount/Amount/text()"))
        GetRate.InsideDelivery = Val(Xml.ReadXPathFromText(fdxRate.Response, fdxStr(2) & "Surcharges[SurchargeType='INSIDE_DELIVERY']/Amount/Amount/text()"))
        
    Else
        GetRate.ErrorCode = CInt(Val(Xml.ReadXPathFromText(fdxRate.Response, "/RateReply/Notifications/Code/text()")))
        GetRate.ErrorMessage = Xml.ReadXPathFromText(fdxRate.Response, "/RateReply/Notifications/Message/text()")
        MsgBox GetRate.ErrorMessage & " [" & GetRate.ErrorCode & "]", vbCritical, "FedEx Web Services"
    End If
    
End Function

Function GetService(ByVal sFromCountry As String, ByVal sIntcode As String, Optional ByVal sRateOrShip As String = "RATE") As FedExService
    Dim Service As FedExService
    
    Service.LTL = False
    If sFromCountry = "US" Then
        Select Case sIntcode
            Case "ZFP", "XFP"
                Service.Code = "112"
                Service.Type = "FEDEX_FREIGHT_PRIORITY"
                Service.Description = "FedEx Freight Priority"
                Service.LTL = True
            Case "ZFE", "XFE"
                Service.Code = "113"
                Service.Type = "FEDEX_FREIGHT_ECONOMY"
                Service.Description = "FedEx Freight Economy"
                Service.LTL = True
        End Select
    End If
            
    GetService = Service
End Function

Function SendRequest(fdxTransXml As String, fdxRqType As String) As String
    Dim FdxServer As String
    Dim fdxUrl As String
    
    Dim fdxTransID As String
    
    FdxServer = "ws.fedex.com"
    
    Select Case Upper(fdxRqType)
    Case "RATE"
        fdxUrl = "/web-services/rate"
        fdxTransID = "Rate-" & Alltrim(CS32.GetVar("xPKGID"))
    End Select
    
    fdxTransID = Alltrim(fdxTransID)
    fdxTransID = StrTran(fdxTransID, "\", "_")
    fdxTransID = StrTran(fdxTransID, "/", "_")
    fdxTransID = StrTran(fdxTransID, ":", "_")
    fdxTransID = StrTran(fdxTransID, "*", "_")
    fdxTransID = StrTran(fdxTransID, "?", "_")
    fdxTransID = StrTran(fdxTransID, "<", "_")
    fdxTransID = StrTran(fdxTransID, ">", "_")
    fdxTransID = StrTran(fdxTransID, "|", "_")
    
    'Save file
    Common.SaveTextToFile fdxTransXml, fdxTransID, "xml", fdxDir & "XML", CreateUnique
    
    fdxTransXml = StrTran(fdxTransXml, vbTab, "")
    fdxTransXml = StrTran(fdxTransXml, vbLf, "")
    'fdxTransXml = URLEncode(fdxTransXml)
    
    Dim fdxHttp As MSXML2.ServerXMLHTTP60
    Set fdxHttp = New MSXML2.ServerXMLHTTP60
    
    fdxStr(1) = IniRead(fdxIni, "API", "Proxy", "")
    If Not KiEmpty(fdxStr(1)) Then
        fdxHttp.setProxy 2, fdxStr(1)
    End If
    fdxHttp.setOption 2, 13056
    fdxHttp.setTimeouts 30000, 30000, 30000, 30000
    fdxHttp.open "POST", "https://" & FdxServer & fdxUrl, False
    fdxHttp.setRequestHeader "SOAPAction", """http://fedex.com/ws/rate/v22/getRates"""
    fdxHttp.setRequestHeader "Host", FdxServer & ":443"
    fdxHttp.setRequestHeader "Connection", "Keep-Alive"
    fdxHttp.setRequestHeader "User-Agent", "Mozilla/4.0"
    fdxHttp.setRequestHeader "Content-Type", "text/xml; charset=utf-8"
    fdxHttp.Send "<?xml version=""1.0"" encoding=""utf-8""?>" & fdxTransXml
    fdxStr(0) = fdxHttp.responseText
    
    Do While InStr(fdxStr(0), "xmlns:") > 0
        fdxInt(0) = InStr(fdxStr(0), "xmlns:")
        fdxStr(1) = SubStr(fdxStr(0), fdxInt(0))
        fdxStr(1) = Left(fdxStr(1), InStr(fdxStr(1), ">") - 1)
        
        fdxStr(2) = SubStr(fdxStr(1), Len("xmlns:") + 1)
        fdxStr(2) = Left(fdxStr(2), InStr(fdxStr(2), "=") - 1)
        
        fdxStr(0) = StrTran(fdxStr(0), "<" & fdxStr(2) & ":", "<")
        fdxStr(0) = StrTran(fdxStr(0), "</" & fdxStr(2) & ":", "</")
        
        fdxStr(0) = StrTran(fdxStr(0), fdxStr(1), "")
    Loop
    
    Do While InStr(fdxStr(0), "xmlns=") > 0
        fdxInt(0) = InStr(fdxStr(0), "xmlns=")
        fdxStr(1) = SubStr(fdxStr(0), fdxInt(0))
        fdxStr(1) = Left(fdxStr(1), InStr(fdxStr(1), ">") - 1)
        
        fdxStr(0) = StrTran(fdxStr(0), fdxStr(1), "")
    Loop
    
    fdxStr(0) = SubStr(fdxStr(0), InStr(fdxStr(0), "<Body>") + Len("<Body>"))
    fdxStr(0) = Left(fdxStr(0), InStr(fdxStr(0), "</Body>") - 1)
   
    SendRequest = fdxStr(0)
    
    'Save response to file.
    Common.SaveTextToFile fdxStr(0), fdxTransID & "-response", "xml", fdxDir & "XML", CreateUnique
    KillStatus
    
    fdxStr(1) = Xml.ReadXPathFromText(fdxStr(0), "/Fault/detail/fault/errorCode/text()")
    If Not KiEmpty(fdxStr(1)) Then
        fdxStr(2) = Xml.ReadXPathFromText(fdxStr(0), "/Fault/detail/fault/reason/text()")
        MsgBox fdxStr(2) & " [" & fdxStr(1) & "]", vbExclamation, "FedEx Web Services Error"
    End If
End Function

