Attribute VB_Name = "UpsAPI"
Dim upsInitialized As Boolean
Dim upsApiEnabled As Boolean

Dim RateUpsIntCodes As Boolean
Dim ShipUpsIntCodes As Boolean

Dim upsApiUsername As String
Dim upsApiPassword As String
Dim upsApiKey As String

Dim upsTntRq As String, upsTntRs As String
Dim upsRateRq As String, upsRateRs As String
Dim upsServer As String
Dim upsDir As String, upsTmpDir As String, upsIni As String

Dim upsHttp As MSXML2.ServerXMLHTTP60
Dim upsResponseXml As MSXML2.DOMDocument60

Dim upsStr(10) As String, upsInt(10) As Integer, upsDbl(10) As Double, upsBool(10) As Boolean, upsLong(10) As Long, upsVar(10) As Variant

Dim UpsRateTrans As UpsTransaction
Dim UpsShipTrans As UpsTransaction

Enum RateOption
    Rate = 1
    Shop = 2
    Ratetimeintransit = 3
    Shoptimeintransit = 4
End Enum
Public Enum UpsRateType
    Base
    Total
End Enum

Type UpsService
    Code As String
    Description As String
    LTL As Boolean
    GroundFreightPrice As Boolean
End Type
Type UpsAddress
    AccountNumber As String
    Name As String
    AddressLine1 As String
    AddressLine2 As String
    AddressLine3 As String
    City As String
    State As String
    PostalCode As String
    CountryCode As String
    Phone As String
    EmailAddress As String
    Residential As Boolean
End Type

Type UpsTransaction
    Request As String
    Response As String
End Type

Type UpsRateResult
    Status As String
    ErrorCode As Integer
    ErrorMessage As String
    Success As Boolean
    Base As Double
    Fuel As Double
    Total As Double
End Type
Type UpsShipResult
    Status As String
    TrackingNumber As String
    BOLID As String
    Origin As String
    ErrorCode As Integer
    ErrorMessage As String
    Success As Boolean
    Base As Double
    Fuel As Double
    Total As Double
    LabelFile As String
    BOLFile As String
End Type

    
Function Initialize()
    Initialize = upsInitialized
    If upsInitialized Then
        Exit Function
    End If
    
    upsDir = CS32.ShipDir & "Logic\UPSAPI"
    
    If Dir(upsDir, vbDirectory) = "" Then
        MkDir upsDir
    End If
    upsDir = upsDir & "\"
        
    If Dir(upsDir & "XML", vbDirectory) = "" Then
        MkDir upsDir & "XML"
    End If
    
    upsTmpDir = ProjectDir & "Temp"
    If Dir(upsTmpDir, vbDirectory) = "" Then
        MkDir upsTmpDir
    End If
    upsTmpDir = upsTmpDir & "\"
    
    upsIni = upsDir & "UPSAPI.ini"
    
    upsStr(0) = Left(Upper(IniRead(upsIni, "Settings", "Enabled", "")), 1)
    If KiEmpty(upsStr(0)) Then
        upsStr(0) = "N"
        IniWrite upsIni, "Settings", "Enabled", upsStr(0)
    End If
    upsApiEnabled = (upsStr(0) = "Y")
    
    If upsApiEnabled Then
        upsApiUsername = IniRead(upsIni, "Settings", "Username", "")
        upsApiPassword = IniRead(upsIni, "Settings", "Password", "")
        upsApiKey = IniRead(upsIni, "Settings", "Key", "")
        
        If KiEmpty(upsApiUsername) Or KiEmpty(upsApiPassword) Or KiEmpty(upsApiKey) Then
            MsgBox "UPS OnLine settings missing. UPS OnLine will be disabled.", vbInformation
            upsApiEnabled = False
            Exit Function
        End If
        
        RateUpsIntCodes = (IniRead(upsIni, "Settings", "EnableRatingForUpsIntCodes", "Y") = "Y")
        ShipUpsIntCodes = (IniRead(upsIni, "Settings", "EnableShippingForUpsIntCodes", "N") = "Y")
    End If
    
    upsInitialized = True
    Initialize = upsInitialized
End Function

Public Function GetRate(Optional ByVal ShowStatus As Boolean = True) As UpsRateResult
    upsInitialized = Initialize()
    If Not upsInitialized Then
        Exit Function
    End If
    
    If ShowStatus Then PopStatus "UPS Online Tools", "Getting rates for UPS, please wait..."
    
    GetRate.Success = False
    
    Dim UpsSvc As UpsService
    UpsSvc = GetService("US", CS32.Intcode, "RATE")
    If KiEmpty(UpsSvc.Code) Then
        Exit Function
    End If
    
    upsStr(0) = ""
    If UpsSvc.LTL Then
       upsStr(0) = CreateRateRequest_Freight(UpsSvc)
    Else
       upsStr(0) = CreateRateRequest(UpsSvc)
    End If
    
    If UpsRateTrans.Request <> upsStr(0) Then
        UpsRateTrans.Response = ""
    End If
        
    upsStr(1) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/Response/ResponseStatus/Code/text()")
    If (upsStr(1) <> "1") Then
        UpsRateTrans.Request = upsStr(0)
        
        upsStr(1) = "RATE"
        If UpsSvc.LTL Then
            upsStr(1) = "RATEFREIGHT"
        End If
            
        UpsRateTrans.Response = SendRequest(UpsRateTrans.Request, upsStr(1))
    End If

    Dim sDelDate As String
    
    If UpsSvc.GroundFreightPrice Then
        Dim dblSvcOptChg As Double, dblFrsChg As Double
        
        upsStr(3) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment/ServiceOptionsCharges/MonetaryValue/text()")
        If Not KiEmpty(upsStr(3)) Then
            dblSvcOptChg = CDbl(upsStr(3))
        End If
        
        upsStr(4) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment/FRSShipmentData/TransportationCharges/NetCharge/MonetaryValue/text()")
        If Not KiEmpty(upsStr(4)) Then
            dblFrsChg = CDbl(upsStr(4))
        End If
        
        GetRate.Success = True
        GetRate.Base = dblFrsChg
        GetRate.Total = dblFrsChg + dblSvcOptChg
    ElseIf UpsSvc.LTL Then
        GetRate.Success = True
        GetRate.Base = Val(Xml.ReadXPathFromText(UpsRateTrans.Response, "/FreightRateResponse/Rate[Type/Code='AFTR_DSCNT']/Factor/Value/text()"))
        GetRate.Total = Val(Xml.ReadXPathFromText(UpsRateTrans.Response, "/FreightRateResponse/TotalShipmentCharge/MonetaryValue/text()"))
        GetRate.Fuel = Val(Xml.ReadXPathFromText(UpsRateTrans.Response, "/FreightRateResponse/Rate[Type/Code='2']/Factor/Value/text()"))
    Else
        Dim dblDiscChg As Double
        dblDiscChg = CDbl(Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment[Service/Code='" & UpsService.Code & "']/NegotiatedRateCharges/TotalCharge/MonetaryValue/text()"))
        GetRate.Total = dblDiscChg
        GetRate.Status = True
    End If
    
    sDelDate = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment[Service/Code='" & UpsSvc.Code & "']/TimeInTransit/ServiceSummary/EstimatedArrival/Arrival/Date/text()")
    CS32.SetVar "xVDC_PHONE", sDelDate
End Function

Public Function Ship() As UpsShipResult
    Ship.Success = False
    
    upsInitialized = Initialize()
    If Not upsInitialized Then
        Exit Function
    End If
    
    Dim UpsSvc As UpsService
    UpsSvc = GetService("US", CS32.Intcode, "SHIP")
    If KiEmpty(UpsSvc.Code) Then
        Exit Function
    End If
    
    'Turn off CS label.
    CS32.SetVar "xUSE_LABEL", "000000000"
    
    PopStatus "UPS Online Tools", "Sending to UPS..."
    
    upsStr(0) = ""
    If UpsSvc.LTL Then
       upsStr(0) = CreateShipRequest_Freight(UpsSvc)
    Else
       'upsStr(0) = CreateShipRequest(UpsSvc)
    End If
    
    If UpsShipTrans.Request <> upsStr(0) Then
        UpsShipTrans.Response = ""
    End If
        
    upsStr(1) = Xml.ReadXPathFromText(UpsShipTrans.Response, "/ShipResponse/Response/ResponseStatus/Code/text()")
    If (upsStr(1) <> "1") Then
        UpsShipTrans.Request = upsStr(0)
        
        upsStr(1) = "SHIP"
        If UpsSvc.LTL Then
            upsStr(1) = "SHIPFREIGHT"
        End If
            
        UpsShipTrans.Response = SendRequest(UpsShipTrans.Request, upsStr(1))
    End If

    Dim sDelDate As String
    
    If UpsSvc.GroundFreightPrice Then
        Dim dblSvcOptChg As Double, dblFrsChg As Double
        
        upsStr(3) = Xml.ReadXPathFromText(UpsShipTrans.Response, "/ShipResponse/RatedShipment/ServiceOptionsCharges/MonetaryValue/text()")
        If Not KiEmpty(upsStr(3)) Then
            dblSvcOptChg = CDbl(upsStr(3))
        End If
        
        upsStr(4) = Xml.ReadXPathFromText(UpsShipTrans.Response, "/ShipResponse/RatedShipment/FRSShipmentData/TransportationCharges/NetCharge/MonetaryValue/text()")
        If Not KiEmpty(upsStr(4)) Then
            dblFrsChg = CDbl(upsStr(4))
        End If
        
        GetRate.Success = True
        GetRate.Base = dblFrsChg
        GetRate.Total = dblFrsChg + dblSvcOptChg
    ElseIf UpsSvc.LTL Then
        Ship.TrackingNumber = Xml.ReadXPathFromText(UpsShipTrans.Response, "/FreightShipResponse/ShipmentResults/ShipmentNumber/text()")
        Ship.BOLID = Xml.ReadXPathFromText(UpsShipTrans.Response, "/FreightShipResponse/ShipmentResults/BOLID/text()")
        Ship.Base = Val(Xml.ReadXPathFromText(UpsShipTrans.Response, "/FreightShipResponse/ShipmentResults/Rate[Type/Code='AFTR_DSCNT']/Factor/Value/text()"))
        Ship.Total = Val(Xml.ReadXPathFromText(UpsShipTrans.Response, "/FreightShipResponse/ShipmentResults/TotalShipmentCharge/MonetaryValue/text()"))
        Ship.Fuel = Val(Xml.ReadXPathFromText(UpsShipTrans.Response, "/FreightShipResponse/ShipmentResults/Rate[Type/Code='2']/Factor/Value/text()"))
        
        upsStr(1) = Xml.ReadXPathFromText(UpsShipTrans.Response, "//ShipmentResults/Documents/Image[Type/Code='21']/GraphicImage/text()")
        If Not KiEmpty(upsStr(1)) Then
            'Save base64 data to temp file.
            Common.SaveTextToFile upsStr(1), "ups_bol", "b64", upsTmpDir, Overwrite
        
            'Decode base64 file to new file.
            Common.Base64_Decode upsTmpDir & "ups_bol.b64", upsTmpDir & "ups_bol.pdf"
                        
            'Move file to docs folder.
            If fExist(upsTmpDir & "ups_bol.pdf") Then
                fMove upsTmpDir & "ups_bol.pdf", upsDir & "Docs\BOL_" & Ship.BOLID & ".pdf"
            End If
            
            PrintBOL Ship.BOLID
        End If
        
        CS32.SetVar "xSHP_CHG", Ship.Base
        CS32.SetVar "xTOT_CHG", Ship.Total
        CS32.SetVar "xFSC_CHG", Ship.Fuel
        CS32.SetVar "xTRACK_NO", Ship.TrackingNumber
        CS32.SetVar "xTRACK_MOM", Ship.BOLID
        CS32.SetVar "xSHP_WGT", Xml.ReadXPathFromText(UpsShipTrans.Response, "//ShipmentResults/BillableShipmentWeight/Value/text()")
        
        Ship.Success = True
    Else
        Dim dblDiscChg As Double
        dblDiscChg = CDbl(Xml.ReadXPathFromText(UpsShipTrans.Response, "/ShipResponse/RatedShipment[Service/Code='" & UpsService.Code & "']/NegotiatedRateCharges/TotalCharge/MonetaryValue/text()"))
        GetRate.Total = dblDiscChg
        GetRate.Status = True
    End If
    
    sDelDate = Xml.ReadXPathFromText(UpsShipTrans.Response, "/RateResponse/RatedShipment[Service/Code='" & UpsSvc.Code & "']/TimeInTransit/ServiceSummary/EstimatedArrival/Arrival/Date/text()")
    CS32.SetVar "xVDC_PHONE", sDelDate
End Function


Private Function CreateRateRequest(ByRef UpsSvc As UpsService) As String
    Dim sRqOption As String
    sRqOption = "Shoptimeintransit"
    If UpsSvc.GroundFreightPrice Then
        sRqOption = "Rate"
    End If
    
    upsStr(0) = "<?xml version=""1.0"" encoding=""utf-8""?>" & vbLf & _
        "<soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">" & vbLf & _
        vbTab & "<soap:Header>" & vbLf & _
        vbTab & vbTab & "<UPSSecurity xmlns=""http://www.ups.com/XMLSchema/XOLTWS/UPSS/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Username", upsApiUsername) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Password", upsApiPassword) & vbLf & _
        vbTab & vbTab & vbTab & "</UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & "<ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccessLicenseNumber", upsApiKey) & vbLf & _
        vbTab & vbTab & vbTab & "</ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & "</UPSSecurity>" & vbLf & _
        vbTab & "</soap:Header>" & vbLf

    upsStr(0) = upsStr(0) & _
        vbTab & "<soap:Body>" & vbLf & _
        vbTab & vbTab & "<RateRequest xmlns=""http://www.ups.com/XMLSchema/XOLTWS/Rate/v1.1"">" & vbLf & _
        vbTab & vbTab & vbTab & "<Request xmlns=""http://www.ups.com/XMLSchema/XOLTWS/Common/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("RequestOption", sRqOption) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<SubVersion>1701</SubVersion>" & vbLf & _
        vbTab & vbTab & vbTab & "</Request>" & vbLf & _
        vbTab & vbTab & vbTab & "<CustomerClassification>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "00") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Rates Associated with Shipper Number") & vbLf & _
        vbTab & vbTab & vbTab & "</CustomerClassification>" & vbLf

    upsInt(1) = CInt(CS32.GetVar("xACCOUNT"))
    
    Dim shipper As UpsAddress
    shipper.AccountNumber = IniRead(upsIni, "Account" & upsInt(1), "ShipperNumber", "")
    shipper.Name = IniRead(upsIni, "Account" & upsInt(1), "ShipperName", "")
    shipper.AddressLine1 = IniRead(upsIni, "Account" & upsInt(1), "ShipperAddress1", "")
    shipper.City = IniRead(upsIni, "Account" & upsInt(1), "ShipperCity", "")
    shipper.State = IniRead(upsIni, "Account" & upsInt(1), "ShipperState", "")
    shipper.PostalCode = IniRead(upsIni, "Account" & upsInt(1), "ShipperPostalCode", "")
    shipper.CountryCode = IniRead(upsIni, "Account" & upsInt(1), "ShipperCountry", "US")

    upsBool(1) = KiEmpty(shipper.AccountNumber)
    upsBool(2) = KiEmpty(shipper.Name & shipper.AddressLine1 & shipper.City & shipper.State & shipper.PostalCode)
    If upsBool(1) Or upsBool(2) Then
        Dim kdvAcctInfo As KiDataView
        Set kdvAcctInfo = DbfSelect(CS32.ShipDir & "CS_ACCT.FIL", "RECNO()=" & upsInt(1))
        If Not kdvAcctInfo.EOF Then
            If upsBool(1) Then
                shipper.AccountNumber = StrTran(SubStr(Trim(kdvAcctInfo("ACCTNUMUPS")), 4), "-", "")
            End If
            If upsBool(2) Then
                shipper.Name = Trim(kdvAcctInfo("FROM1"))
                shipper.AddressLine1 = Trim(kdvAcctInfo("FROM2"))
                shipper.City = Trim(kdvAcctInfo("FROMCITY"))
                shipper.State = Trim(kdvAcctInfo("FROMSTATE"))
                shipper.PostalCode = Trim(kdvAcctInfo("FROMZIP"))
            End If
        End If
        Set kdvAcctInfo = Nothing
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & "<Shipment>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Shipper>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", shipper.AccountNumber) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", shipper.AddressLine1) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", shipper.City) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", shipper.State) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", shipper.PostalCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", shipper.CountryCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Shipper>" & vbLf

    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<ShipFrom>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", shipper.AddressLine1) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", shipper.City) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", shipper.State) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", shipper.PostalCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", shipper.CountryCode) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</ShipFrom>" & vbLf

    upsStr(1) = Alltrim(CS32.GetVar("xSHIPTO7"))
    upsStr(2) = ""
    
    On Error Resume Next
    If SubStr(upsStr(1), Len(upsStr(1)) - 2, 1) = " " Then
        upsStr(2) = Right(upsStr(1), 2)
        upsStr(1) = Left(upsStr(1), Len(upsStr(1)) - 2)
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<ShipTo>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", Alltrim(CS32.GetVar("xSHIPTO4"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(1)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(2)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<CountryCode>US</CountryCode>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</ShipTo>" & vbLf

    If UpsSvc.GroundFreightPrice Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & "<FRSPaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Prepaid") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</FRSPaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Service>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", UpsSvc.Code) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", UpsSvc.Description) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Service>" & vbLf
    End If
    
    upsStr(1) = ""
    upsStr(2) = ""
    'If CS32.GetVar("xUSE_ICE") = "Y" Then
    '    upsStr(2) = "" & _
    '        "<DryIce>" & vbLf & _
    '        vbTab & Xml.WriteTag("RegulationSet", "CFR") & vbLf & _
    '        vbTab & "<DryIceWeight>" & vbLf & _
    '        vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
    '        vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
    '        vbTab & vbTab & vbTab & Xml.WriteTag("Description", "LBS") & vbLf & _
    '        vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
    '        vbTab & vbTab & Xml.WriteTag("Weight", Trim(CS32.GetVar("xICE_WGT"))) & vbLf & _
    '        vbTab & "</DryIceWeight>" & vbLf & _
    '        "</DryIce>" & vbLf
    'End If
            
    If Not KiEmpty(upsStr(2)) Then
        upsStr(1) = "" & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<PackageServiceOptions>" & vbLf & _
            upsStr(2) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</PackageServiceOptions>"
    End If
    
    Dim kdvPkgInfo As KiDataView
    Set kdvPkgInfo = DbfSelect(ProjectDir & "PKG_INFO.DBF", "PKGID='" & PadR(Trim(CS32.GetVar("xPKGID")), 20) & "'")
    If Not kdvPkgInfo.EOF Then
        Do While Not kdvPkgInfo.EOF
            For i = 1 To CInt(kdvPkgInfo("PKG_QTY"))
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & "<Package>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<PackagingType>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>02</Code>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</PackagingType>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<Dimensions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>IN</Code>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Length", SubToken(Trim(kdvPkgInfo("DIMENSIONS")), 1, "x")) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Width", SubToken(Trim(kdvPkgInfo("DIMENSIONS")), 2, "x")) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Height", SubToken(Trim(kdvPkgInfo("DIMENSIONS")), 3, "x")) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</Dimensions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<PackageWeight>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>LBS</Code>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Weight", Trim(kdvPkgInfo("ACT_WGT"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</PackageWeight>" & vbLf
                    
                If UpsSvc.GroundFreightPrice Then
                    upsStr(0) = upsStr(0) & _
                        Replicate(vbTab, 5) & "<Commodity>" & vbLf & _
                        Replicate(vbTab, 6) & Xml.WriteTag("FreightClass", Trim(kdvPkgInfo("CLASS"))) & vbLf & _
                        Replicate(vbTab, 5) & "</Commodity>" & vbLf
                End If
                    
                upsStr(0) = upsStr(0) & _
                    upsStr(1) & _
                    vbTab & vbTab & vbTab & vbTab & "</Package>" & vbLf
            Next
            kdvPkgInfo.MoveNext
        Loop
    Else
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & "<Package>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<PackagingType>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>02</Code>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</PackagingType>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<Dimensions>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>IN</Code>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Length", Alltrim(CS32.GetVar("xDIM_LENGTH"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Width", Alltrim(CS32.GetVar("xDIM_WIDTH"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Height", Alltrim(CS32.GetVar("xDIM_HEIGHT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</Dimensions>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<PackageWeight>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Code>LBS</Code>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Weight", Alltrim(CS32.GetVar("xACT_WGT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</PackageWeight>" & vbLf & _
            upsStr(1) & _
            vbTab & vbTab & vbTab & vbTab & "</Package>" & vbLf
    
    End If
    Set kdvPkgInfo = Nothing

    upsStr(1) = ""
    If UpsSvc.GroundFreightPrice Then
        upsStr(1) = vbTab & vbTab & vbTab & vbTab & vbTab & "<FRSShipmentIndicator />" & vbLf
    End If
        
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<ShipmentRatingOptions>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<NegotiatedRatesIndicator />" & vbLf & _
        upsStr(1) & _
        vbTab & vbTab & vbTab & vbTab & "</ShipmentRatingOptions>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<DeliveryTimeInformation>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<PackageBillType>03</PackageBillType>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</DeliveryTimeInformation>" & vbLf & _
        vbTab & vbTab & vbTab & "</Shipment>" & vbLf & _
        vbTab & vbTab & "</RateRequest>" & vbLf & _
        vbTab & "</soap:Body>" & vbLf & _
        "</soap:Envelope>" & vbLf
        
    CreateRateRequest = upsStr(0)
End Function

Function GetRatex(ByVal sCsIntcode As String, Optional ByRef ReturnRate As UpsRateType = UpsRateType.Total) As Double
    upsInitialized = Initialize()
    If Not (upsInitialized And upsApiEnabled) Then
        Exit Function
    End If

    Dim UpsService As UpsService
    
    UpsService = GetService("US", sCsIntcode, "RATE")
    If KiEmpty(UpsService.Code) Then
        Exit Function
    End If
    
    upsStr(0) = CreateRateRequest(UpsService)
    If UpsRateTrans.Request <> upsStr(0) Then
        UpsRateTrans.Response = ""
    End If
        
    upsStr(1) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/Response/ResponseStatus/Code/text()")
    If (upsStr(1) <> "1") Then
        UpsRateTrans.Request = upsStr(0)
        UpsRateTrans.Response = SendRequest(UpsRateTrans.Request, "Rate")
    End If

    Dim sDelDate As String
    
    If UpsService.GroundFreightPrice Then
        Dim dblSvcOptChg As Double, dblFrsChg As Double
        
        upsStr(3) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment/ServiceOptionsCharges/MonetaryValue/text()")
        If Not KiEmpty(upsStr(3)) Then
            dblSvcOptChg = CDbl(upsStr(3))
        End If
        
        upsStr(4) = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment/FRSShipmentData/TransportationCharges/NetCharge/MonetaryValue/text()")
        If Not KiEmpty(upsStr(4)) Then
            dblFrsChg = CDbl(upsStr(4))
        End If
        
        If ReturnRate = UpsRateType.Base Then
            GetRatex = dblFrsChg
        Else
            GetRatex = dblFrsChg + dblSvcOptChg
        End If
    Else
        Dim dblDiscChg As Double
        dblDiscChg = CDbl(Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment[Service/Code='" & UpsService.Code & "']/NegotiatedRateCharges/TotalCharge/MonetaryValue/text()"))
        GetRatex = dblDiscChg
        'If dblDiscChg > 0 Then
        '    Dim dblTotChg As Double, dblDiscAmt As Double
        '    dblTotChg = CDbl(CS32.GetVar("xTOT_CHG"))
        '
        '    dblDiscAmt = dblTotChg - dblDiscChg
        '
        '    CS32.SetVar "xINP_CHG", dblDiscAmt
        '    CS32.SetVar "xTOT_CHG", dblDiscChg
        'End If
    End If
    
    sDelDate = Xml.ReadXPathFromText(UpsRateTrans.Response, "/RateResponse/RatedShipment[Service/Code='" & UpsService.Code & "']/TimeInTransit/ServiceSummary/EstimatedArrival/Arrival/Date/text()")
    CS32.SetVar "xVDC_PHONE", sDelDate
End Function

Sub Ship_Freight()
    Dim csIntcode As String
    Dim upsUrl As String, upsServer As String
    
    Dim uoltTransXml As String, uoltResponseXml As String
    
    csIntcode = CS32.GetVar("xINTCODE")
    
    Dim UpsSvc As UpsService
    Dim sRateRqOption As String
    
    UpsSvc = GetService("US", Trim(CS32.GetVar("xINTCODE")), "SHIP")
    
    If KiEmpty(UpsSvc.Code) Or Not UpsSvc.LTL Then
        Exit Sub
    End If
    
    UpsAPI.CreateShipment_Freight "SHIP"
    
    upsStr(1) = "FreightShipRequest"
    
    uoltTransXml = "" & _
        "<?xml version=""1.0"" encoding=""utf-8""?>" & vbLf & _
        "<soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" " & _
        "xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" " & _
        "xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">" & vbLf & _
        vbTab & "<soap:Header>" & vbLf & _
        vbTab & vbTab & "<UPSSecurity xmlns=""http://www.ups.com/XMLSchema/XOLTWS/UPSS/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Username", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Access/UserId/text()")) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Password", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Access/Password/text()")) & vbLf & _
        vbTab & vbTab & vbTab & "</UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & "<ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccessLicenseNumber", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Access/AccessKey/text()")) & vbLf & _
        vbTab & vbTab & vbTab & "</ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & "</UPSSecurity>" & vbLf & _
        vbTab & "</soap:Header>" & vbLf
        
    uoltTransXml = uoltTransXml & _
        vbTab & "<soap:Body>" & vbLf & _
        vbTab & vbTab & "<FreightShipRequest xmlns=""http://www.ups.com/XMLSchema/XOLTWS/FreightShip/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<Request xmlns=""http://www.ups.com/XMLSchema/XOLTWS/Common/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<RequestOption>1</RequestOption>" & vbLf & _
        vbTab & vbTab & vbTab & "</Request>" & vbLf & _
        uoltShipmentXML & _
        vbTab & vbTab & "</FreightShipRequest>" & vbLf & _
        vbTab & "</soap:Body>" & vbLf & _
        "</soap:Envelope>"
    
    upsStr(1) = uoltDir & "XML\"
    upsStr(2) = Common.SaveTextToFile(uoltTransXml, "Conf-" & Alltrim(CS32.GetVar("xPKGID")), "xml", upsStr(1), CreateUnique)
    
    uoltTransXml = StrTran(uoltTransXml, vbTab, "")
    uoltTransXml = StrTran(uoltTransXml, vbLf, "")
    
    upsUrl = "webservices/FreightShip"

    Set upsHttp = New MSXML2.ServerXMLHTTP60
    upsHttp.setOption 2, 13056
    upsHttp.setTimeouts 30000, 30000, 30000, 30000
    upsHttp.open "POST", "https://" & upsServer & upsUrl, False
    upsHttp.setRequestHeader "Content-Type", "application/x-www-form-urlencoded"
    upsHttp.Send uoltTransXml
    uoltResponseXml = upsHttp.responseText
    
    XN1 = InStr(uoltResponseXml, "<?")
    If XN1 > 0 Then
        XC2 = SubStr(uoltResponseXml, XN1)
        XN1 = InStr(XC2, ">")
        XC2 = Left(XC2, XN1 + 1)
        uoltResponseXml = StrTran(uoltResponseXml, XC2, "")
    End If
    
    XN1 = InStr(uoltResponseXml, "<")
    If XN1 > 0 Then
        uoltResponseXml = SubStr(uoltResponseXml, XN1)
        XN1 = InStr(uoltResponseXml, ">")
        XC2 = Left(uoltResponseXml, XN1 + 1)
        XN1 = InStr(XC2, " ")
        If XN1 > 0 Then
            uoltResponseXml = StrTran(uoltResponseXml, XC2, Left(XC2, XN1 - 1) & ">")
        End If
    End If
    
    upsStr(3) = Common.SaveTextToFile(uoltResponseXml, upsStr(2) & "-response", "xml", upsStr(1), CreateUnique)
    
    uoltResponseXml = StrTran(uoltResponseXml, "<soapenv:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</soapenv:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<freightShip:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</freightShip:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<common:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</common:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<err:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</err:", "</")
    
    'uoltXmlShipRs = xmlReadXSL(uoltResponseXml, "/Envelope/Body/*")
    
    upsInt(1) = InStr(uoltResponseXml, "<Body")
    If upsInt(1) > 0 Then
        uoltResponseXml = SubStr(uoltResponseXml, upsInt(1) + 5)
        upsInt(1) = InStr(uoltResponseXml, ">")
        uoltResponseXml = SubStr(uoltResponseXml, upsInt(1) + 1)
        
        upsInt(1) = InStr(uoltResponseXml, "</Body>")
        uoltResponseXml = Left(uoltResponseXml, upsInt(1) - 1)
    End If
    
    upsStr(4) = Xml.ReadXPathFromText(uoltResponseXml, "/FreightShipResponse/Response/ResponseStatus/Code/text()")
    If upsStr(4) <> "1" Then
        upsStr(4) = Xml.ReadXPathFromText(uoltResponseXml, "/Fault/detail/Errors/ErrorDetail/PrimaryErrorCode/Description/text()")
        If KiEmpty(upsStr(4)) Then
            upsStr(4) = "Error processing UPS API transaction"
        End If
        CS32.SetVar "xERROR", upsStr(4)
        Exit Sub
    End If
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/ShipmentNumber/text()")
    CS32.SetVar "xTRACK_NO", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/BOLID/text()")
    CS32.SetVar "xTRACK_MOM", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/Rate[Type/Code='AFTR_DSCNT']/Factor/Value/text()")
    CS32.SetVar "xSHP_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/Rate[Type/Code='2']/Factor/Value/text()")
    CS32.SetVar "xFSC_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/TotalShipmentCharge/MonetaryValue/text()")
    CS32.SetVar "xTOT_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/BillableShipmentWeight/Value/text()")
    CS32.SetVar "xSHP_WGT", upsStr(6)

End Sub

Function CreateRateRequest_Freight(ByRef UpsSvc As UpsService)
    Dim iCsAcct As Integer
    
    Dim upsUrl As String, upsServer As String
    
    Dim uoltTransXml As String, uoltResponseXml As String
    
    sCsIntcode = CS32.GetVar("xINTCODE")
    iCsAcct = CInt(CS32.GetVar("xACCOUNT"))
    
    If KiEmpty(UpsSvc.Code) Then
        Exit Function
    End If
    
    If Not UpsSvc.LTL Then
        Exit Function
    End If
    
    upsStr(1) = "FreightRateRequest"
    
    upsStr(0) = "" & _
        "<?xml version=""1.0"" encoding=""utf-8""?>" & vbLf & _
        "<soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" " & _
        "xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" " & _
        "xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">" & vbLf & _
        vbTab & "<soap:Header>" & vbLf & _
        vbTab & vbTab & "<UPSSecurity xmlns=""http://www.ups.com/XMLSchema/XOLTWS/UPSS/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Username", upsApiUsername) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Password", upsApiPassword) & vbLf & _
        vbTab & vbTab & vbTab & "</UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & "<ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccessLicenseNumber", upsApiKey) & vbLf & _
        vbTab & vbTab & vbTab & "</ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & "</UPSSecurity>" & vbLf & _
        vbTab & "</soap:Header>" & vbLf
        
    upsStr(0) = upsStr(0) & _
        vbTab & "<soap:Body>" & vbLf & _
        vbTab & vbTab & "<FreightRateRequest xmlns=""http://www.ups.com/XMLSchema/XOLTWS/FreightRate/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<Request xmlns=""http://www.ups.com/XMLSchema/XOLTWS/Common/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("RequestOption", "1") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SubVersion", "1707") & vbLf & _
        vbTab & vbTab & vbTab & "</Request>" & vbLf
        
    'Ship from
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<ShipFrom>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xFROM1"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", Alltrim(CS32.GetVar("xFROM2"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", Alltrim(CS32.GetVar("xFROMCITY"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", Alltrim(CS32.GetVar("xFROMSTATE"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xFROMZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", IniRead(upsIni, "Account1", "ShipperContact", "Shipping Dept")) & vbLf & _
        vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", Alltrim(CS32.GetVar("xFROM4"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
        vbTab & vbTab & "</ShipFrom>" & vbLf & _
    ""

    'ShipTo
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<ShipTo>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xSHIPTO1"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Address>" & vbLf

    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO4"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO5"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO6"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO7"))
    upsStr(4) = Common.GetCity(upsStr(3))
    upsStr(3) = Common.GetState(upsStr(3))
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xSHIPTO3"))) & vbLf

    upsStr(5) = Alltrim(CS32.GetVar("xPHONE"))
    If KiEmpty(upsStr(5)) Then
        upsStr(5) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/DefaultShipToPhone/text()")
    End If
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", upsStr(5)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", Alltrim(CS32.GetVar("xCONSEMAIL"))) & vbLf & _
        vbTab & vbTab & "</ShipTo>" & vbLf
    
    'End ShipTo
    
    'Payment Info
    upsInt(1) = Max(Val(CS32.GetVar("xPAY_FLAG")), 1)
    If upsInt(1) <= 1 Then
        upsStr(2) = ""
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress1", Trim(CS32.GetVar("xFROM2")))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress2", "")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress3", "")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", IniRead(upsIni, "Account" & iCsAcct, "ShipperCompany", Trim(CS32.GetVar("xFROM1")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
            upsStr(2) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", IniRead(upsIni, "Account" & iCsAcct, "ShipperCity", Trim(CS32.GetVar("xFROMCITY")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperState", Trim(CS32.GetVar("xFROMSTATE")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperZip", Trim(CS32.GetVar("xFROMZIP")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperCountry", "US")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", uoltShipperNo) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", IniRead(upsIni, "Account" & iCsAcct, "ShipperContact", "Shipping Dept")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", IniRead(upsIni, "Account" & iCsAcct, "ShipperPhone", Trim(CS32.GetVar("xFROM4")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", IniRead(upsIni, "Account" & iCsAcct, "ShipperEmail", "")) & vbLf & _
            vbTab & vbTab & vbTab & "</Payer>" & vbLf & _
            vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "10") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "PREPAID") & vbLf & _
            vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & "</PaymentInformation>" & vbLf
    Else
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xBILLTO1"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf

        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO3"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO4"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO5"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO6"))
        upsStr(4) = Right(upsStr(3), 2)
        upsStr(3) = Alltrim(Left(upsStr(3), Len(upsStr(3)) - 2))
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf
            
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xBILLTO7"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", Alltrim(CS32.GetVar("xPAY_ACCT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xBILLTO4"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
            vbTab & vbTab & vbTab & "</Payer>" & vbLf

        If upsInt(1) = 3 Then
            upsStr(5) = "30|THIRD PARTY"
        Else
            upsStr(5) = "40|FREIGHT COLLECT"
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", SubToken(upsStr(5), 1, "|")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", SubToken(upsStr(5), 2, "|")) & vbLf & _
            vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & "</PaymentInformation>" & vbLf
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<Service>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Code", UpsSvc.Code) & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Description", UpsSvc.Description) & vbLf & _
        vbTab & vbTab & "</Service>" & vbLf & _
        ""

    'Build line items xml.
    Dim kdvBlData As KiDataView
    Set kdvBlData = DbfSelect(ProjectDir & "BL_DATA.DBF", , , 0)
    
    If Not kdvBlData.EOF Then
        upsStr(1) = Trim(kdvBlData("HND_TYPE"))
        If upsStr(1) = "SKD" Or upsStr(1) = "SKID" Then
            upsStr(1) = "SKD"
            upsStr(2) = "SKID"
        ElseIf upsStr(1) = "TOT" Or upsStr(1) = "TOTE" Or upsStr(1) = "TOTES" Then
            upsStr(1) = "TOT"
            upsStr(2) = "TOTES"
        ElseIf upsStr(1) = "CBY" Or upsStr(1) = "CARBOY" Then
            upsStr(1) = "CBY"
            upsStr(2) = "CARBOY"
        Else
            upsStr(1) = "PLT"
            upsStr(2) = "PALLET"
        End If
    
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<HandlingUnitOne>" & vbLf & _
            vbTab & vbTab & vbTab & Xml.WriteTag("Quantity", Trim(kdvBlData("HND_QTY"))) & vbLf & _
            vbTab & vbTab & vbTab & "<Type>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", upsStr(1)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", upsStr(2)) & vbLf & _
            vbTab & vbTab & vbTab & "</Type>" & vbLf & _
            vbTab & vbTab & "</HandlingUnitOne>" & vbLf
    
        If Not kdvBlData.EOF Then
            upsStr(3) = Trim(kdvBlData("SS_FLAGS"))
        
            While Not kdvBlData.EOF
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & "<Commodity>" & vbLf & _
                    vbTab & vbTab & vbTab & Xml.WriteTag("Description", "COMMODITY DESCRIPTION") & vbLf & _
                    vbTab & vbTab & vbTab & "<Weight>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "lbs") & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "pounds") & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Value", Trim(kdvBlData("WEIGHT"))) & vbLf & _
                    vbTab & vbTab & vbTab & "</Weight>" & vbLf & _
                    vbTab & vbTab & vbTab & Xml.WriteTag("FreightClass", Trim(kdvBlData("CLASS"))) & vbLf & _
                    vbTab & vbTab & vbTab & Xml.WriteTag("NumberOfPieces", Trim(kdvBlData("PKG_QTY"))) & vbLf & _
                    vbTab & vbTab & vbTab & "<PackagingType>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", Trim(kdvBlData("PKG_TYPE"))) & vbLf & _
                    vbTab & vbTab & vbTab & "</PackagingType>" & vbLf & _
                    vbTab & vbTab & "</Commodity>" & vbLf
                
                kdvBlData.MoveNext
            Wend
            
            If InStr(upsStr(3), "Y") > 0 Then
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<ShipmentServiceOptions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<DeliveryOptions>" & vbLf
                
                If SubStr(upsStr(3), 1, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<CallBeforeDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 4, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<LimitedAccessDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 11, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<LiftGateRequiredIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 12, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<ResidentialDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 18, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<InsideDeliveryIndicator />" & vbLf
                End If
                
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</DeliveryOptions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</ShipmentServiceOptions>" & vbLf
            End If
        End If
    End If
    Set kdvBlData = Nothing


    upsStr(0) = upsStr(0) & _
        uoltShipmentXML & _
        vbTab & vbTab & "</FreightRateRequest>" & vbLf & _
        vbTab & "</soap:Body>" & vbLf & _
        "</soap:Envelope>"
    
    CreateRateRequest_Freight = upsStr(0)
    
End Function

Function CreateShipRequest_Freight(ByRef UpsSvc As UpsService)
    Dim iCsAcct As Integer
    
    Dim upsUrl As String, upsServer As String
    
    Dim uoltTransXml As String, uoltResponseXml As String
    
    iCsAcct = CInt(CS32.GetVar("xACCOUNT"))
    
    If KiEmpty(UpsSvc.Code) Then
        Exit Function
    End If
    
    If Not UpsSvc.LTL Then
        Exit Function
    End If
    
    upsStr(1) = "FreightShipRequest"
    
    upsStr(0) = "" & _
        "<?xml version=""1.0"" encoding=""utf-8""?>" & vbLf & _
        "<soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" " & _
        "xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" " & _
        "xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">" & vbLf & _
        vbTab & "<soap:Header>" & vbLf & _
        vbTab & vbTab & "<UPSSecurity xmlns=""http://www.ups.com/XMLSchema/XOLTWS/UPSS/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Username", upsApiUsername) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Password", upsApiPassword) & vbLf & _
        vbTab & vbTab & vbTab & "</UsernameToken>" & vbLf & _
        vbTab & vbTab & vbTab & "<ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AccessLicenseNumber", upsApiKey) & vbLf & _
        vbTab & vbTab & vbTab & "</ServiceAccessToken>" & vbLf & _
        vbTab & vbTab & "</UPSSecurity>" & vbLf & _
        vbTab & "</soap:Header>" & vbLf
        
    upsStr(0) = upsStr(0) & _
        vbTab & "<soap:Body>" & vbLf & _
        vbTab & vbTab & "<FreightShipRequest xmlns=""http://www.ups.com/XMLSchema/XOLTWS/FreightShip/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & "<Request xmlns=""http://www.ups.com/XMLSchema/XOLTWS/Common/v1.0"">" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<RequestOption>1</RequestOption>" & vbLf & _
        vbTab & vbTab & vbTab & "</Request>" & vbLf

    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & "<Shipment>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<RateInformation>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<NegotiatedRatesIndicator />" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</RateInformation>" & vbLf
        
    'Ship from
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<ShipFrom>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xFROM1"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", Alltrim(CS32.GetVar("xFROM2"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", Alltrim(CS32.GetVar("xFROMCITY"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", Alltrim(CS32.GetVar("xFROMSTATE"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xFROMZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", IniRead(upsIni, "Account1", "ShipperContact", "Shipping Dept")) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", Alltrim(CS32.GetVar("xFROM4"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</ShipFrom>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", IniRead(upsIni, "Account" & iCsAcct, "Account", StrTran(SubStr(CS32.GetVar("xACCTNUMUPS"), 4), "-", ""))) & vbLf & _
        ""

    'ShipTo
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<ShipTo>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xSHIPTO1"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf

    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO4"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO5"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO6"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO7"))
    upsStr(4) = Right(upsStr(3), 2)
    upsStr(3) = Alltrim(Left(upsStr(3), Len(upsStr(3)) - 2))
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xSHIPTO3"))) & vbLf

    upsStr(5) = Alltrim(CS32.GetVar("xPHONE"))
    If KiEmpty(upsStr(5)) Then
        upsStr(5) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/DefaultShipToPhone/text()")
    End If
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", upsStr(5)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", Alltrim(CS32.GetVar("xCONSEMAIL"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</ShipTo>" & vbLf
    
    'End ShipTo
    
            
   'Payment Info
    If iCsAcct <= 1 Then
        upsStr(2) = ""
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress1", Trim(CS32.GetVar("xFROM2")))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress2", "")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = IniRead(upsIni, "Account" & iCsAcct, "ShipperAddress3", "")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", IniRead(upsIni, "Account" & iCsAcct, "ShipperCompany", Trim(CS32.GetVar("xFROM1")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
            upsStr(2) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", IniRead(upsIni, "Account" & iCsAcct, "ShipperCity", Trim(CS32.GetVar("xFROMCITY")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperState", Trim(CS32.GetVar("xFROMSTATE")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperZip", Trim(CS32.GetVar("xFROMZIP")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", IniRead(upsIni, "Account" & iCsAcct, "ShipperCountry", "US")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", uoltShipperNo) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", IniRead(upsIni, "Account" & iCsAcct, "ShipperContact", "Shipping Dept")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", IniRead(upsIni, "Account" & iCsAcct, "ShipperPhone", Trim(CS32.GetVar("xFROM4")))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", IniRead(upsIni, "Account" & iCsAcct, "ShipperEmail", "")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "10") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "PREPAID") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</PaymentInformation>" & vbLf
    Else
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xBILLTO1"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf

        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO3"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO4"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO5"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(0) = upsStr(0) & _
                vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO6"))
        upsStr(4) = Right(upsStr(3), 2)
        upsStr(3) = Alltrim(Left(upsStr(3), Len(upsStr(3)) - 2))
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf
            
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xBILLTO7"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", Alltrim(CS32.GetVar("xPAY_ACCT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xBILLTO2"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", Trim(CS32.GetVar("xBILLTO5"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</Payer>" & vbLf

        If upsInt(1) = 3 Then
            upsStr(5) = "30|THIRD PARTY"
        Else
            upsStr(5) = "40|FREIGHT COLLECT"
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", SubToken(upsStr(5), 1, "|")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", SubToken(upsStr(5), 2, "|")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</PaymentInformation>" & vbLf
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<Service>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", UpsSvc.Code) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", UpsSvc.Description) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Service>" & vbLf & _
        ""

    'Build line items xml.
    Dim kdvBlData As KiDataView
    Set kdvBlData = DbfSelect(ProjectDir & "BL_DATA.DBF", , , 0)
    
    If Not kdvBlData.EOF Then
        upsStr(1) = Trim(kdvBlData("HND_TYPE"))
        If upsStr(1) = "SKD" Or upsStr(1) = "SKID" Then
            upsStr(1) = "SKD"
            upsStr(2) = "SKID"
        ElseIf upsStr(1) = "TOT" Or upsStr(1) = "TOTE" Or upsStr(1) = "TOTES" Then
            upsStr(1) = "TOT"
            upsStr(2) = "TOTES"
        ElseIf upsStr(1) = "CBY" Or upsStr(1) = "CARBOY" Then
            upsStr(1) = "CBY"
            upsStr(2) = "CARBOY"
        Else
            upsStr(1) = "PLT"
            upsStr(2) = "PALLET"
        End If
    
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & vbTab & "<HandlingUnitOne>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Quantity", Trim(kdvBlData("HND_QTY"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", upsStr(1)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", upsStr(2)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</HandlingUnitOne>" & vbLf
    
        If Not kdvBlData.EOF Then
            upsStr(3) = Trim(kdvBlData("SS_FLAGS"))
        
            While Not kdvBlData.EOF
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & "<Commodity>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", Trim(kdvBlData("ITEMDESC"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<Weight>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "lbs") & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "pounds") & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Value", Trim(kdvBlData("WEIGHT"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</Weight>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("FreightClass", Trim(kdvBlData("CLASS"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("NumberOfPieces", Trim(kdvBlData("PKG_QTY"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "<PackagingType>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", Trim(kdvBlData("PKG_TYPE"))) & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & "</PackagingType>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & "</Commodity>" & vbLf
                
                kdvBlData.MoveNext
            Wend
            
            If InStr(upsStr(3), "Y") > 0 Then
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<ShipmentServiceOptions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<DeliveryOptions>" & vbLf
                
                If SubStr(upsStr(3), 1, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<CallBeforeDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 4, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<LimitedAccessDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 16, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<LiftGateRequiredIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 12, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<ResidentialDeliveryIndicator />" & vbLf
                End If
                If SubStr(upsStr(3), 18, 1) = "Y" Then
                    upsStr(0) = upsStr(0) & _
                        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<InsideDeliveryIndicator />" & vbLf
                End If
                
                upsStr(0) = upsStr(0) & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</DeliveryOptions>" & vbLf & _
                    vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</ShipmentServiceOptions>" & vbLf
            End If
        End If
    End If
    Set kdvBlData = Nothing

    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & "<Documents>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Image>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "30") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Label") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("LabelsPerPage", "1") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "PDF") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<PrintFormat>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "02") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Thermal") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</PrintFormat>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<PrintSize>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Width", "6") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Length", "4") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</PrintSize>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Image>" & vbLf & _
        ""

    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "<Image>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "21") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "VICS BOL") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "<Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "PDF") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & vbTab & "</Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & "</Image>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Documents>" & vbLf & _
        ""
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & "</Shipment>" & vbLf & _
        vbTab & vbTab & "</FreightShipRequest>" & vbLf & _
        vbTab & "</soap:Body>" & vbLf & _
        "</soap:Envelope>"
    
    CreateShipRequest_Freight = upsStr(0)
End Function


Function CreateShipment_Freight(sAction As String) As String
    Dim kdvBlData As KiDataView
    Set kdvBlData = DbfSelect(ProjectDir & "BL_DATA.DBF", "1=1", , 0)

    upsStr(1) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/TaxId/text()")
    If KiEmpty(upsStr(1)) Then
        upsStr(1) = IniRead(ShipDir() & "UPS\UPS.ini", "Account" & csAcct, "CustomsID", "")
    End If
    
    upsStr(0) = "" & _
        vbTab & "<Shipment>" & vbLf & _
        vbTab & vbTab & "<RateInformation>" & vbLf & _
        vbTab & vbTab & vbTab & "<NegotiatedRatesIndicator />" & vbLf & _
        vbTab & vbTab & "</RateInformation>" & vbLf & _
        vbTab & vbTab & "<Description />" & vbLf
    
    '============
    'Ship from.
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<ShipFrom>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xFROM1"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", Alltrim(CS32.GetVar("xFROM2"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", Alltrim(CS32.GetVar("xFROMCITY"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", Alltrim(CS32.GetVar("xFROMSTATE"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xFROMZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Name/text()")) & vbLf & _
        vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", Alltrim(CS32.GetVar("xFROM4"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
        vbTab & vbTab & "</ShipFrom>" & vbLf & _
        vbTab & vbTab & Xml.WriteTag("ShipperNumber", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/AccountNo/text()")) & vbLf
    
    '=========
    'Ship to
    
    upsStr(2) = ""
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO4"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO5"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO6"))
    If Not KiEmpty(upsStr(3)) Then
        upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3)) & vbLf
    End If
    
    upsStr(3) = Alltrim(CS32.GetVar("xSHIPTO7"))
    upsStr(4) = Right(upsStr(3), 2)
    upsStr(3) = Alltrim(Left(upsStr(3), Len(upsStr(3)) - 2))
        
    upsStr(5) = Alltrim(CS32.GetVar("xPHONE"))
    If KiEmpty(upsStr(5)) Then
        upsStr(5) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/DefaultShipToPhone/text()")
    End If
        
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<ShipTo>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xSHIPTO1"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Address>" & vbLf & _
        upsStr(2) & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xZIP"))) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
        vbTab & vbTab & vbTab & "</Address>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xSHIPTO3"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", upsStr(5)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
        vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", Alltrim(CS32.GetVar("xCONSEMAIL"))) & vbLf & _
        vbTab & vbTab & "</ShipTo>" & vbLf
    
    
    
    upsInt(1) = Max(Val(CS32.GetVar("xPAY_FLAG")), 1)
    If upsInt(1) <= 1 Then
        upsStr(2) = ""
        upsStr(3) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Address1/text()")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Address2/text()")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Address3/text()")
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Company/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
            upsStr(2) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/City/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/StateProvince/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/PostalCode/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Country/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", uoltShipperNo) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Name/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Phone/text()")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", Xml.ReadXPathFromText(uoltCfgXml, "/UPSOLT/Accounts/Account[@id='" & csAcct & "']/Shipper/Email/text()")) & vbLf & _
            vbTab & vbTab & vbTab & "</Payer>" & vbLf & _
            vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "10") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "PREPAID") & vbLf & _
            vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & "</PaymentInformation>" & vbLf
    Else
        upsStr(2) = ""
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO3"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO4"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO5"))
        If Not KiEmpty(upsStr(3)) Then
            upsStr(2) = upsStr(2) & vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AddressLine", upsStr(3))
        End If
        
        upsStr(3) = Alltrim(CS32.GetVar("xBILLTO6"))
        upsStr(4) = Right(upsStr(3), 2)
        upsStr(3) = Alltrim(Left(upsStr(3), Len(upsStr(3)) - 2))
        
        If upsInt(1) = 3 Then
            upsStr(5) = "30|THIRD PARTY"
        Else
            upsStr(5) = "40|FREIGHT COLLECT"
        End If
        
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<PaymentInformation>" & vbLf & _
            vbTab & vbTab & vbTab & "<Payer>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Name", Alltrim(CS32.GetVar("xBILLTO1"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Address>" & vbLf & _
            upsStr(2) & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("City", upsStr(3)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("StateProvinceCode", upsStr(4)) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PostalCode", Alltrim(CS32.GetVar("xBILLTO7"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CountryCode", "US") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Address>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("ShipperNumber", Alltrim(CS32.GetVar("xPAY_ACCT"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("AttentionName", Alltrim(CS32.GetVar("xBILLTO4"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Number", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Extension", "") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</Phone>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("EMailAddress", "") & vbLf & _
            vbTab & vbTab & vbTab & "</Payer>" & vbLf & _
            vbTab & vbTab & vbTab & "<ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", SubToken(upsStr(5), 1, "|")) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", SubToken(upsStr(5), 2, "|")) & vbLf & _
            vbTab & vbTab & vbTab & "</ShipmentBillingOption>" & vbLf & _
            vbTab & vbTab & "</PaymentInformation>" & vbLf
    End If
    
    '=========
    'Service.
    If sAction <> "SHIP" Then
        sAction = "RATE"
    End If
    
    'UPS API can use different service codes based on from-country -> to-country combo, and whether rating or shipping.
    upsStr(4) = "/UPSOLT/Services/Service[@intcode='" & csIntcode & "']/"
    upsStr(5) = "Code[((@origin='US' or not(@origin)) or (contains(@origins, 'US;') or not(@origins))) and (@use='" & sAction & "' or not(@use))]/"
    upsStr(1) = Xml.ReadXPathFromText(uoltCfgXml, upsStr(4) + upsStr(5) & "text()")
    'Debug(upsStr(4) + upsStr(5) & " = " & upsStr(1))
    If KiEmpty(upsStr(1)) Then
        CS32.SetError "Unsupported internal code."
        Exit Function
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<Service>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Code", upsStr(1)) & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Description", Xml.ReadXPathFromText(uoltCfgXml, str3 & "Description/text()")) & vbLf & _
        vbTab & vbTab & "</Service>" & vbLf

    
    upsStr(1) = Xml.ReadXPathFromText(ltlData, "/blShipment/items/item[1]/hnduom/text()")
    upsStr(1) = Trim(kdvBlData("HND_TYPE"))
    If upsStr(1) = "SKD" Or upsStr(1) = "SKID" Then
        upsStr(1) = "SKD"
        upsStr(2) = "SKID"
    ElseIf upsStr(1) = "TOT" Or upsStr(1) = "TOTE" Or upsStr(1) = "TOTES" Then
        upsStr(1) = "TOT"
        upsStr(2) = "TOTES"
    ElseIf upsStr(1) = "CBY" Or upsStr(1) = "CARBOY" Then
        upsStr(1) = "CBY"
        upsStr(2) = "CARBOY"
    Else
        upsStr(1) = "PLT"
        upsStr(2) = "PALLET"
    End If
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<HandlingUnitOne>" & vbLf & _
        vbTab & vbTab & vbTab & Xml.WriteTag("Quantity", Trim(kdvBlData("HND_QTY"))) & vbLf & _
        vbTab & vbTab & vbTab & "<Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", upsStr(1)) & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", upsStr(2)) & vbLf & _
        vbTab & vbTab & vbTab & "</Type>" & vbLf & _
        vbTab & vbTab & "</HandlingUnitOne>" & vbLf

    upsInt(0) = 1
    Do While Not kdvBlData.EOF
        upsStr(1) = "/blShipment/items/item[" & Alltrim(Str(upsInt(0), -1, 0)) & "]/"
        upsStr(1) = Trim(kdvBlData("PKG_QTY"))
        'Debug(str0 & "pkgqty/text() => " & upsStr(1))
        If Val(upsStr(1)) <= 0 Then
            Exit Do
        End If
        
        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & "<Commodity>" & vbLf & _
            vbTab & vbTab & vbTab & "<Weight>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "<UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "lbs") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "pounds") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & "</UnitOfMeasurement>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Value", Trim(kdvBlData("WEIGHT"))) & vbLf & _
            vbTab & vbTab & vbTab & "</Weight>" & vbLf & _
            vbTab & vbTab & vbTab & Xml.WriteTag("NumberOfPieces", Trim(kdvBlData("PKG_QTY"))) & vbLf

        upsStr(0) = upsStr(0) & _
            vbTab & vbTab & vbTab & "<PackagingType>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "PKG") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Package") & vbLf & _
            vbTab & vbTab & vbTab & "</PackagingType>" & vbLf & _
            vbTab & vbTab & vbTab & "<CommodityValue>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("CurrencyCode", "USD") & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("MonetaryValue", Trim(kdvBlData("BL_ITVAL"))) & vbLf & _
            vbTab & vbTab & vbTab & "</CommodityValue>" & vbLf & _
            vbTab & vbTab & vbTab & Xml.WriteTag("FreightClass", Trim(kdvBlData("CLASS"))) & vbLf & _
            vbTab & vbTab & vbTab & "<NMFCCommodity>" & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("PrimeCode", Trim(kdvBlData("NMFC"))) & vbLf & _
            vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("SubCode", "") & vbLf & _
            vbTab & vbTab & vbTab & "</NMFCCommodity>" & vbLf & _
            vbTab & vbTab & "</Commodity>" & vbLf
            
        upsInt(0) = upsInt(0) + 1
        kdvldata.MoveNext
    Loop
    
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & "<Documents>" & vbLf & _
        vbTab & vbTab & vbTab & "<Image>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "30") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Label") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("LabelsPerPage", "1") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Pdf") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<PrintFormat>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "02") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Thermal") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</PrintFormat>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<PrintSize>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Length", "6") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Width", "4") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</PrintSize>" & vbLf & _
        vbTab & vbTab & vbTab & "</Image>" & vbLf
        
    upsStr(0) = upsStr(0) & _
        vbTab & vbTab & vbTab & "<Image>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "21") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "VICS BOL") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Type>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "<Format>" & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Code", "01") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & vbTab & Xml.WriteTag("Description", "Pdf") & vbLf & _
        vbTab & vbTab & vbTab & vbTab & "</Format>" & vbLf & _
        vbTab & vbTab & vbTab & "</Image>" & vbLf & _
        vbTab & vbTab & "</Documents>" & vbLf

    upsStr(0) = upsStr(0) & vbTab & "</Shipment>" & vbLf

    Set kdvBlData = Nothing
End Function

Function SendRequest(upsTransXml As String, upsRqType As String) As String
    'PopStatus "Endicia Label Server", "Sending " & elsXmlPrefix
    Dim upsServer As String
    Dim upsUrl As String
    
    Dim upsTransID As String
    
    upsServer = "wwwcie.ups.com"
    upsServer = "onlinetools.ups.com"
    
    Select Case Upper(upsRqType)
    Case "RATE"
        upsUrl = "/webservices/Rate"
        upsTransID = "Rate-" & Alltrim(CS32.GetVar("xPKGID"))
    Case "RATEFREIGHT"
        upsUrl = "/webservices/FreightRate"
        upsTransID = "RateFreight-" & Alltrim(CS32.GetVar("xPKGID"))
    Case "SHIP"
        upsUrl = "/webservices/Ship"
        upsTransID = "Rate-" & Alltrim(CS32.GetVar("xPKGID"))
    Case "SHIPFREIGHT"
        upsUrl = "/webservices/FreightShip"
        upsTransID = "ShipFreight-" & Alltrim(CS32.GetVar("xPKGID"))
    Case "TIMEINTRANSIT"
        upsUrl = "/webservices/TimeInTransit"
        upsTransID = "TimeInTransit-" & Alltrim(CS32.GetVar("xPKGID"))
    End Select
    
    upsTransID = Alltrim(upsTransID)
    upsTransID = StrTran(upsTransID, "\", "_")
    upsTransID = StrTran(upsTransID, "/", "_")
    upsTransID = StrTran(upsTransID, ":", "_")
    upsTransID = StrTran(upsTransID, "*", "_")
    upsTransID = StrTran(upsTransID, "?", "_")
    upsTransID = StrTran(upsTransID, "<", "_")
    upsTransID = StrTran(upsTransID, ">", "_")
    upsTransID = StrTran(upsTransID, "|", "_")
    
    'Save file
    Common.SaveTextToFile upsTransXml, upsTransID, "xml", upsDir & "XML", CreateUnique
    
    upsTransXml = StrTran(upsTransXml, vbTab, "")
    upsTransXml = StrTran(upsTransXml, vbLf, "")
    'upsTransXml = URLEncode(upsTransXml)
    
    Dim upsHttp As MSXML2.ServerXMLHTTP60
    Set upsHttp = New MSXML2.ServerXMLHTTP60
    
    upsStr(1) = IniRead(upsIni, "Settings", "Proxy", "")
    If Not KiEmpty(upsStr(1)) Then
        upsHttp.setProxy 2, upsStr(1)
    End If
    upsHttp.setOption 2, 13056
    upsHttp.setTimeouts 30000, 30000, 30000, 30000
    
    upsHttp.open "POST", "https://" & upsServer & upsUrl, False
    upsHttp.setRequestHeader "Content-Type", "application/x-www-form-urlencoded"
    upsHttp.Send upsTransXml
    uoltResponseXml = upsHttp.responseText
    
    Common.SaveTextToFile uoltResponseXml, upsTransID & "-responseraw", "xml", upsDir & "XML", CreateUnique
    
    upsStr(0) = uoltResponseXml
    Do While InStr(upsStr(0), "xmlns:") > 0
        upsLong(0) = InStr(upsStr(0), "xmlns:")
        upsStr(1) = SubStr(upsStr(0), upsLong(0))
        upsStr(1) = Left(upsStr(1), InStr(upsStr(1), ">") - 1)
        
        upsStr(2) = SubStr(upsStr(1), Len("xmlns:") + 1)
        upsStr(2) = Left(upsStr(2), InStr(upsStr(2), "=") - 1)
        
        upsStr(0) = StrTran(upsStr(0), "<" & upsStr(2) & ":", "<")
        upsStr(0) = StrTran(upsStr(0), "</" & upsStr(2) & ":", "</")
        
        upsStr(0) = StrTran(upsStr(0), upsStr(1), "")
    Loop
    uoltResponseXml = upsStr(0)
    
    upsLong(1) = InStr(uoltResponseXml, "<?")
    If upsLong(1) > 0 Then
        upsStr(2) = SubStr(uoltResponseXml, upsLong(1))
        upsLong(1) = InStr(upsStr(2), ">")
        upsStr(2) = Left(upsStr(2), upsLong(1) + 1)
        uoltResponseXml = StrTran(uoltResponseXml, upsStr(2), "")
    End If
    
    upsLong(1) = InStr(uoltResponseXml, "<")
    If upsLong(1) > 0 Then
        uoltResponseXml = SubStr(uoltResponseXml, upsLong(1))
        upsLong(1) = InStr(uoltResponseXml, ">")
        upsStr(2) = Left(uoltResponseXml, upsLong(1) + 1)
        upsLong(1) = InStr(upsStr(2), " ")
        If upsLong(1) > 0 Then
            uoltResponseXml = StrTran(uoltResponseXml, upsStr(2), Left(upsStr(2), upsLong(1) - 1) & ">")
        End If
    End If
    
    uoltResponseXml = StrTran(uoltResponseXml, "<soapenv:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</soapenv:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<freightShip:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</freightShip:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<common:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</common:", "</")
    uoltResponseXml = StrTran(uoltResponseXml, "<err:", "<")
    uoltResponseXml = StrTran(uoltResponseXml, "</err:", "</")
    
    'uoltXmlShipRs = xmlReadXSL(uoltResponseXml, "/Envelope/Body/*")
    
    upsLong(1) = InStr(uoltResponseXml, "<Body")
    If upsLong(1) > 0 Then
        uoltResponseXml = SubStr(uoltResponseXml, upsLong(1) + 5)
        upsLong(1) = InStr(uoltResponseXml, ">")
        uoltResponseXml = SubStr(uoltResponseXml, upsLong(1) + 1)
        
        upsLong(1) = InStr(uoltResponseXml, "</Body>")
        uoltResponseXml = Left(uoltResponseXml, upsLong(1) - 1)
    End If
    
    SendRequest = uoltResponseXml
    
    'Save response to file.
    Common.SaveTextToFile uoltResponseXml, upsTransID & "-response", "xml", upsDir & "XML", CreateUnique
    KillStatus
    
    upsStr(1) = Xml.ReadXPathFromText(uoltResponseXml, "/Fault/detail/Errors/ErrorDetail/PrimaryErrorCode/Code/text()")
    If Not KiEmpty(upsStr(1)) Then
        MsgBox Xml.ReadXPathFromText(uoltResponseXml, "/Fault/detail/Errors/ErrorDetail/PrimaryErrorCode/Description/text()") & " [" & upsStr(1) & "]", vbExclamation, "UPS OnLine Tools Error"
    End If

'//////////////////////////////////////////////////
    Exit Function
    
    upsStr(4) = Xml.ReadXPathFromText(uoltResponseXml, "/FreightShipResponse/Response/ResponseStatus/Code/text()")
    If upsStr(4) <> "1" Then
        upsStr(4) = Xml.ReadXPathFromText(uoltResponseXml, "/Fault/detail/Errors/ErrorDetail/PrimaryErrorCode/Description/text()")
        If KiEmpty(upsStr(4)) Then
            upsStr(4) = "Error processing UPS API transaction"
        End If
        CS32.SetVar "xERROR", upsStr(4)
        Exit Function
    End If
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/ShipmentNumber/text()")
    CS32.SetVar "xTRACK_NO", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/BOLID/text()")
    CS32.SetVar "xTRACK_MOM", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/Rate[Type/Code='AFTR_DSCNT']/Factor/Value/text()")
    CS32.SetVar "xSHP_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/Rate[Type/Code='2']/Factor/Value/text()")
    CS32.SetVar "xFSC_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/TotalShipmentCharge/MonetaryValue/text()")
    CS32.SetVar "xTOT_CHG", upsStr(6)
    
    upsStr(6) = Xml.ReadXPathFromText(uoltResponseXml, "//FreightShipResponse/ShipmentResults/BillableShipmentWeight/Value/text()")
    CS32.SetVar "xSHP_WGT", upsStr(6)
End Function

Function GetServiceCode(ByVal sFromCountry As String, ByVal sIntcode As String, Optional ByVal sRateOrShip As String = "RATE") As String
    Dim sSvcCode As String
    Dim Service As UpsService
    
    Service.LTL = False
    If sFromCountry = "US" Then
        Select Case sIntcode
            Case "U01", "U02"
                sSvcCode = "01"
                Service.Code = "01"
                Service.Description = "Next Day Air"
            Case "U07", "U08"
                Service.Code = "02"
            Case "U11"
                Service.Code = "03"
            Case "U21"
                Service.Code = "12"
            Case "U35", "U36"
                Service.Code = "59"
            Case "U43", "U44"
                Service.Code = "13"
            Case "U60", "U61"
                Service.Code = "14"
            Case "U48", "U75"
                Service.Code = "11"
            Case "U49", "U64", "U65", "U68"
                Service.Code = "07"
            Case "U54"
                Service.Code = "08"
            Case "U63", "U66", "U67", "U69"
                Service.Code = "54"
            Case "U71", "U72", "U73", "U74"
                Service.Code = "65"
                
            Case "ZUF", "XUF"
                Service.Code = "308"
                Service.Description = "Freight LTL"
                Service.LTL = True
        End Select
    End If
            
    GetServiceCode = Service.Code
End Function

Function GetService(ByVal sFromCountry As String, ByVal sIntcode As String, Optional ByVal sRateOrShip As String = "RATE") As UpsService
    Dim Service As UpsService
    
    If (sRateOrShip = "RATE" And Left(sIntcode, 1) = "U" And Not RateUpsIntCodes) Then
        Exit Function
    End If
    If (sRateOrShip = "SHIP" And Left(sIntcode, 1) = "U" And Not ShipUpsIntCodes) Then
        Exit Function
    End If
    
    Service.LTL = False
    Service.GroundFreightPrice = False
    If sFromCountry = "US" Then
        Select Case sIntcode
            Case "U01", "U02"
                Service.Code = "01"
                Service.Description = "Next Day Air"
            Case "U07", "U08"
                Service.Code = "02"
                Service.Description = "2nd Day Air"
            Case "U11"
                Service.Code = "03"
                Service.Description = "Ground"
            Case "U76"
                Service.Code = "03"
                Service.Description = "Ground"
                Service.GroundFreightPrice = True
            Case "U21"
                Service.Code = "12"
                Service.Description = "3 Day Select"
            Case "U35", "U36"
                Service.Code = "59"
                Service.Description = "2nd Day Air A.M."
            Case "U43", "U44"
                Service.Code = "13"
                Service.Description = "Next Day Air Saver"
            Case "U60", "U61"
                Service.Code = "14"
                Service.Description = "Next Day Air Early"
            Case "U48", "U75"
                Service.Code = "11"
                Service.Description = "Standard"
            Case "U49", "U64", "U65", "U68"
                Service.Code = "07"
                Service.Code = "Worldwide Express"
            Case "U54"
                Service.Code = "08"
                Service.Code = "Worldwide Expedited"
            Case "U63", "U66", "U67", "U69"
                Service.Code = "54"
                Service.Code = "Worldwide Express Plus"
            Case "U71", "U72", "U73", "U74"
                Service.Code = "65"
                Service.Code = "Worldwide Saver"
            Case "ZUF", "XUF"
                Service.Code = "308"
                Service.Description = "Freight LTL"
                Service.LTL = True
            Case "ZUL", "XUL"
                Service.Code = "309"
                Service.Description = "Freight LTL Guaranteed"
                Service.LTL = True
            Case "ZUA", "XUA"
                Service.Code = "308"
                Service.Description = "Freight LTL Guaranteed AM"
                Service.LTL = True
        End Select
    End If
            
    GetService = Service
End Function

Function Check(ByVal sCsIntcode As String) As Boolean
    Check = False
    
    If KiEmpty(UpsAPI.GetService("US", sCsIntcode, "RATE").Code) Then
        Exit Function
    End If
    
    Check = True
End Function

Function PrintBOL(ByVal BOLID As String) As Boolean
    Common.PrintPdf upsDir & "Docs\BOL_" & BOLID & ".pdf", 2
End Function
